import os
import sys
import urllib.parse

try:
    from fuzzywuzzy import fuzz
except ImportError:
    print('[*] fuzzywuzzy not found.')
    print('[*] Please run the script in the setup directory!')
    sys.exit()


def process_group(
        data, group, toc, toc_table, page_num, section,
        sectionid, html):
    """Retreives a group from the full data, and creates toc stuff

    Args:
        data (List): Full set of data containing all hosts
        group (String): String representing group to process
        toc (String): HTML for Table of Contents
        toc_table (String): HTML for Table in ToC
        page_num (int): Page number we're on in the report
        section (String): Display name of the group
        sectionid (String): Unique ID for ToC navigation
        html (String): HTML for current page of report

    Returns:
        List: Elements for category sorted and grouped
        String: HTML representing ToC
        String: HTML representing ToC Table
        String: HTML representing current report page
    """
    group_data = sorted([x for x in data if x.category == group], key=lambda k: str(k.page_title))

    grouped_elements = []
    if len(group_data) == 0:
        return grouped_elements, toc, toc_table, html
    if page_num == 0:
        toc += ("<li><a href=\"report.html#{0}\">{1} (Page 1)</a></li>").format(
            sectionid, section)
    else:
        toc += ("<li><a href=\"report_page{0}.html#{1}\">{2} (Page {0})</a></li>").format(
            str(page_num+1), sectionid, section)

    html += "<h2 id=\"{0}\">{1}</h2>".format(sectionid, section)
    unknowns = [x for x in group_data if x.page_title == 'Unknown']
    group_data = [x for x in group_data if x.page_title != 'Unknown']
    while len(group_data) > 0:
        test_element = group_data.pop(0)
        temp = [x for x in group_data if fuzz.token_sort_ratio(
            test_element.page_title, x.page_title) >= 70]
        temp.append(test_element)
        temp = sorted(temp, key=lambda k: k.page_title)
        grouped_elements.extend(temp)
        group_data = [x for x in group_data if fuzz.token_sort_ratio(
            test_element.page_title, x.page_title) < 70]

    grouped_elements.extend(unknowns)
    toc_table += ("<tr><td>{0}</td><td>{1}</td>").format(section,
                                                         str(len(grouped_elements)))
    return grouped_elements, toc, toc_table, html



def sort_data_and_write(cli_parsed, data):
    """Writes out reports for HTTP objects

    Args:
        cli_parsed (TYPE): CLI Options
        data (TYPE): Full set of data
    """
    # We'll be using this number for our table of contents
    total_results = len(data)
    categories = [('highval', 'High Value Targets', 'highval'),
                  ('dirlist', 'Directory Listings', 'dirlist'),
                  ('cms', 'Content Management System (CMS)', 'cms'),
                  ('idrac', 'IDRAC/ILo/Management Interfaces', 'idrac'),
                  ('nas', 'Network Attached Storage (NAS)', 'nas'),
                  ('comms', 'Communications', 'comms'),
                  ('devops', 'Development Operations', 'devops'),
                  ('secops', 'Security Operations', 'secops'),
                  ('appops', 'Application Operations', 'appops'),
                  ('dataops', 'Data Operations', 'dataops'),
                  ('netdev', 'Network Devices', 'netdev'),
                  ('voip', 'Voice/Video over IP (VoIP)', 'voip'),
                  ('printer', 'Printers', 'printer'),
                  ('infrastructure', 'Infrastructure', 'infrastructure'),
                  (None, 'Uncategorized', 'uncat'),
                  ('construction', 'Under Construction', 'construction'),
                  ('crap', 'Splash Pages', 'crap'),
                  ('empty', 'No Significant Content', 'empty'),
                  ('unauth', '401/403 Unauthorized', 'unauth'),
                  ('notfound', '404 Not Found', 'notfound'),
                  ('successfulLogin', 'Successful Logins', 'successfulLogin'),
                  ('identifiedLogin', 'Identified Logins', 'identifiedLogin'),
                  ('redirector', 'Redirecting Pages', 'redirector'),
                  ('badhost', 'Invalid Hostname', 'badhost'),
                  ('inerror', 'Internal Error', 'inerror'),
                  ('badreq', 'Bad Request', 'badreq'),
                  ('badgw', 'Bad Gateway', 'badgw'),
                  ('serviceunavailable', 'Service Unavailable', 'serviceunavailable'),
                  ]
    if total_results == 0:
        return
    # Initialize stuff we need
    pages = []
    toc = create_report_toc_head(cli_parsed.date, cli_parsed.time)
    toc_table = "<table class=\"table\">"
    web_index_head = create_web_index_head(cli_parsed.date, cli_parsed.time)
    table_head = create_table_head()
    counter = 1
    csv_request_data = "Protocol,Port,Domain,Request Status,Screenshot Path, Source Path"

    # Generate and write json log of requests
    for json_request in data:
        url = urllib.parse.urlparse(json_request._remote_system)

        # Determine protocol
        csv_request_data += "\n" + url.scheme + ","
        if url.port is not None:
            csv_request_data += str(url.port) + ","
        elif url.scheme == 'http':
            csv_request_data += "80,"
        elif url.scheme == 'https':
            csv_request_data += "443,"
        try:
            csv_request_data += url.hostname + ","
        except TypeError:
            print("Error when accessing a target's hostname (it's not existent)")
            print("Possible bad url (improperly formatted) in the URL list.")
            print("Fix your list and re-try. Killing EyeWitness....")
            sys.exit(1)
        if json_request._error_state == None:
            csv_request_data += "Successful,"
        else:
            csv_request_data += json_request._error_state + ","
        csv_request_data += json_request._screenshot_path + ","
        csv_request_data += json_request._source_path

    with open(os.path.join(cli_parsed.d, 'Requests.csv'), 'a') as f:
        f.write(csv_request_data)

    # Pre-filter error entries
    def key_lambda(k):
        if k.error_state is None:
            k.error_state = str(k.error_state)
        if k.page_title is None:
            k.page_title = str(k.page_title)
        return (k.error_state, k.page_title)
    errors = sorted([x for x in data if (x is not None) and (x.error_state is not None)],
                     key=key_lambda)
    data[:] = [x for x in data if x.error_state is None]
    data = sorted(data, key=lambda k: str(k.page_title))
    html = u""
    # Loop over our categories and populate HTML
    for cat in categories:
        grouped, toc, toc_table, html = process_group(
            data, cat[0], toc, toc_table, len(pages), cat[1], cat[2], html)
        if len(grouped) > 0:
            html += table_head
        pcount = 0
        for obj in grouped:
            pcount += 1
            html += obj.create_table_html()
            if (counter % cli_parsed.results == 0) or (counter == (total_results) -1):
                html = (web_index_head + "EW_REPLACEME" + html +
                        "</table><br>")
                pages.append(html)
                html = u""
                if pcount < len(grouped):
                    html += table_head
            counter += 1
        if len(grouped) > 0 and counter - 1 % cli_parsed.results != 0:
            html += "</table><br>"

    # Add our errors here (at the very very end)
    if len(errors) > 0:
        html += '<h2>Errors</h2>'
        html += table_head
        for obj in errors:
            html += obj.create_table_html()
            if (counter % cli_parsed.results == 0) or (counter == (total_results)):
                html = (web_index_head + "EW_REPLACEME" + html +
                        "</table><br>")
                pages.append(html)
                html = u"" + table_head
            counter += 1

    # Close out any stuff thats hanging
    toc += "</ul>"
    toc_table += "<tr><td>Errors</td><td>{0}</td></tr>".format(
        str(len(errors)))
    toc_table += "<tr><th>Total</th><td>{0}</td></tr>".format(total_results)
    toc_table += "</table>"

    if (html != u"") and (counter - total_results != 0):
        html = (web_index_head + "EW_REPLACEME" + html +
                "</table><br>")
        pages.append(html)

    toc = "<center>{0}<br><br>{1}<br><br></center>".format(toc, toc_table)

    if len(pages) == 1:
        with open(os.path.join(cli_parsed.d, 'report.html'), 'a') as f:
            f.write(toc)
            f.write(pages[0].replace('EW_REPLACEME', ''))
            f.write("</body>\n</html>")
    else:
        num_pages = len(pages) + 1
        bottom_text = "\n<center><br>"
        bottom_text += ("<a href=\"report.html\"> Page 1</a>")
        skip_last_dummy = False
        # Generate our header/footer data here
        for i in range(2, num_pages):
            badd_page = "</center>EW_REPLACEME<table border=\"1\">\n        <tr>\n        <th>Web Request Info</th>\n        <th>Web Screenshot</th>\n        </tr></table><br>"
            if badd_page in pages[i-1]:
                skip_last_dummy = True
                pass
            else:
                bottom_text += ("<a href=\"report_page{0}.html\"> Page {0}</a>").format(str(i))
        bottom_text += "</center>\n"
        top_text = bottom_text
        # Generate our next/previous page buttons
        if skip_last_dummy:
            amount = len(pages) - 1
        else:
            amount = len(pages)
        for i in range(0, amount):
            headfoot = "<h3>Page {0}</h3>".format(str(i+1))
            headfoot += "<center>"
            if i == 0:
                headfoot += ("<a href=\"report_page2.html\" id=\"next\"> Next Page "
                             "</a></center>")
            elif i == amount - 1:
                if i == 1:
                    headfoot += ("<a href=\"report.html\" id=\"previous\"> Previous Page "
                                 "</a></center>")
                else:
                    headfoot += ("<a href=\"report_page{0}.html\" id=\"previous\"> Previous Page "
                                 "</a></center>").format(str(i))
            elif i == 1:
                headfoot += ("<a href=\"report.html\" id=\"previous\">Previous Page</a>&nbsp"
                             "<a href=\"report_page{0}.html\" id=\"next\"> Next Page"
                             "</a></center>").format(str(i+2))
            else:
                headfoot += ("<a href=\"report_page{0}.html\" id=\"previous\">Previous Page</a>"
                             "&nbsp<a href=\"report_page{1}.html\" id=\"next\"> Next Page"
                             "</a></center>").format(str(i), str(i+2))
            # Finalize our pages by replacing placeholder stuff and writing out
            # the headers/footers
            pages[i] = pages[i].replace(
                'EW_REPLACEME', headfoot + top_text) + bottom_text + '<br>' + headfoot + '</body></html>'

        # Write out our report to disk!
        if len(pages) == 0:
            return
        with open(os.path.join(cli_parsed.d, 'report.html'), 'a') as f:
            f.write(toc)
            f.write(pages[0])
        write_out = len(pages)
        for i in range(2, write_out + 1):
            bad_page = "<table border=\"1\">\n        <tr>\n        <th>Web Request Info</th>\n        <th>Web Screenshot</th>\n        </tr></table><br>\n<center><br><a "
            badd_page2 = "</center>EW_REPLACEME<table border=\"1\">\n        <tr>\n        <th>Web Request Info</th>\n        <th>Web Screenshot</th>\n        </tr></table><br>"
            if (bad_page in pages[i-1]) or (badd_page2 in pages[i-1]):
                pass
            else:
                with open(os.path.join(cli_parsed.d, 'report_page{0}.html'.format(str(i))), 'w') as f:
                    f.write(pages[i - 1])


def create_web_index_head(date, time):
    """Creates the header for a http report

    Args:
        date (String): Date of report start
        time (String): Time of report start

    Returns:
        String: HTTP Report Start html
    """
    return ("""<html>
        <head>
        <link rel=\"stylesheet\" href=\"https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css\" type=\"text/css\"/>
        <title>EyeWitness Report</title>
        <script src="jquery-1.11.3.min.js"></script>
        <script type="text/javascript">
        function toggleUA(id, url){{
        idi = "." + id;
        $(idi).toggle();
        change = document.getElementById(id);
        if (change.innerHTML.indexOf("expand") > -1){{
            change.innerHTML = "Click to collapse User Agents for " + url;
        }}else{{
            change.innerHTML = "Click to expand User Agents for " + url;
        }}
        }}

        document.onkeydown = function(event){{
            event = event || window.event;
            switch (event.keyCode){{
                case 37:
                    leftArrow();
                    break;
                case 39:
                    rightArrow();
                    break;
            }}
        }};
                
        function leftArrow(){{
            $('#previous')[0].click();
        }};

        function rightArrow(){{
            $('#next')[0].click();
        }};

        </script>
        </head>
        <body>
        <center>
        <center>Report Generated on {0} at {1}</center>""").format(date, time)


def search_index_head():
    return ("""<html>
        <head>
        <link rel=\"stylesheet\" href=\"https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css\" type=\"text/css\"/>
        <title>EyeWitness Report</title>
        <script src="jquery-1.11.3.min.js"></script>
        <script type="text/javascript">
        function toggleUA(id, url){{
        idi = "." + id;
        $(idi).toggle();
        change = document.getElementById(id);
        if (change.innerHTML.indexOf("expand") > -1){{
            change.innerHTML = "Click to collapse User Agents for " + url;
        }}else{{
            change.innerHTML = "Click to expand User Agents for " + url;
        }}
        }}
        </script>
        </head>
        <body>
        <center>
        """)


def create_table_head():
    return ("""<table border=\"1\">
        <tr>
        <th>Web Request Info</th>
        <th>Web Screenshot</th>
        </tr>""")


def create_report_toc_head(date, time):
    return ("""<html>
        <head>
        <title>EyeWitness Report Table of Contents</title>
        </head>
        <h2>Table of Contents</h2>""")


def search_report(cli_parsed, data, search_term):
    pages = []
    web_index_head = search_index_head()
    table_head = create_table_head()
    counter = 1

    data[:] = [x for x in data if x.error_state is None]
    data = sorted(data, key=lambda k: k.page_title)
    html = u""

    # Add our errors here (at the very very end)
    html += '<h2>Results for {0}</h2>'.format(search_term)
    html += table_head
    for obj in data:
        html += obj.create_table_html()
        if counter % cli_parsed.results == 0:
            html = (web_index_head + "EW_REPLACEME" + html +
                    "</table><br>")
            pages.append(html)
            html = u"" + table_head
        counter += 1

    if html != u"":
        html = (web_index_head + html + "</table><br>")
        pages.append(html)

    if len(pages) == 1:
        with open(os.path.join(cli_parsed.d, 'search.html'), 'a') as f:
            f.write(pages[0].replace('EW_REPLACEME', ''))
            f.write("</body>\n</html>")
    else:
        num_pages = len(pages) + 1
        bottom_text = "\n<center><br>"
        bottom_text += ("<a href=\"search.html\"> Page 1</a>")
        # Generate our header/footer data here
        for i in range(2, num_pages):
            bottom_text += ("<a href=\"search_page{0}.html\"> Page {0}</a>").format(
                str(i))
        bottom_text += "</center>\n"
        top_text = bottom_text
        # Generate our next/previous page buttons
        for i in range(0, len(pages)):
            headfoot = "<center>"
            if i == 0:
                headfoot += ("<a href=\"search_page2.html\"> Next Page "
                             "</a></center>")
            elif i == len(pages) - 1:
                if i == 1:
                    headfoot += ("<a href=\"search.html\"> Previous Page "
                                 "</a></center>")
                else:
                    headfoot += ("<a href=\"search_page{0}.html\"> Previous Page "
                                 "</a></center>").format(str(i))
            elif i == 1:
                headfoot += ("<a href=\"search.html\">Previous Page</a>&nbsp"
                             "<a href=\"search_page{0}.html\"> Next Page"
                             "</a></center>").format(str(i+2))
            else:
                headfoot += ("<a href=\"search_page{0}.html\">Previous Page</a>"
                             "&nbsp<a href=\"search_page{1}.html\"> Next Page"
                             "</a></center>").format(str(i), str(i+2))
            # Finalize our pages by replacing placeholder stuff and writing out
            # the headers/footers
            pages[i] = pages[i].replace(
                'EW_REPLACEME', headfoot + top_text) + bottom_text + '<br>' + headfoot + '</body></html>'

        # Write out our report to disk!
        if len(pages) == 0:
            return
        with open(os.path.join(cli_parsed.d, 'search.html'), 'a') as f:
            try:
                f.write(pages[0])
            except UnicodeEncodeError:
                f.write(pages[0].encode('utf-8'))
        for i in range(2, len(pages) + 1):
            with open(os.path.join(cli_parsed.d, 'search_page{0}.html'.format(str(i))), 'w') as f:
                try:
                    f.write(pages[i - 1])
                except UnicodeEncodeError:
                    f.write(pages[i - 1].encode('utf-8'))



'''
Nuclei Functionality, currently limited
'''

import os
import subprocess
from colorama import init, Fore, Style
import pkg_resources


def clear_screen():
    os.system('cls' if os.name == 'nt' else 'clear')


# This will give you the path to where the 'infiltra' package is installed
infiltra_path = pkg_resources.resource_filename('infiltra', '')

# Construct the path to the 'nuclei-templates' directory
nuclei_templates_path = os.path.join(infiltra_path, 'nuclei-templates')
ssl_templates_path = os.path.join(nuclei_templates_path, 'ssl')
fuzzing_templates_path = os.path.join(nuclei_templates_path, 'http/fuzzing/')

# Now you can use this path in your nuclei command
nuclei_command = f"nuclei -t {nuclei_templates_path}"

# Initialize Colorama with autoreset
init(autoreset=True)

# Define colors using Colorama, matching ept.py
DEFAULT_COLOR = Fore.WHITE
BOLD_BLUE = Fore.BLUE + Style.BRIGHT
BOLD_CYAN = Fore.CYAN + Style.BRIGHT
BOLD_GREEN = Fore.GREEN + Style.BRIGHT
BOLD_RED = Fore.RED + Style.BRIGHT
BOLD_YELLOW = Fore.YELLOW + Style.BRIGHT


def is_go_installed():
    try:
        subprocess.run(["go", "version"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return True
    except (subprocess.CalledProcessError, FileNotFoundError):
        return False


def install_go():
    print(f"{BOLD_YELLOW}Updating package list and installing Go...")
    os.system("sudo apt update && sudo apt install -y golang-go")


def setup_go_environment():
    print(f"{BOLD_CYAN}Setting up Go environment...")
    gopath = subprocess.check_output("go env GOPATH", shell=True).decode().strip()
    bashrc_update = f"\nexport GOPATH={gopath}\nexport PATH=$PATH:$GOPATH/bin\n"
    with open(os.path.expanduser("~/.bashrc"), "a") as bashrc:
        bashrc.write(bashrc_update)
    if os.path.exists(os.path.expanduser("~/.zshrc")):
        with open(os.path.expanduser("~/.zshrc"), "a") as zshrc:
            zshrc.write(bashrc_update)
    os.system("source ~/.bashrc")


def install_nuclei():
    print(f"{BOLD_GREEN}Installing Nuclei...")
    os.system("sudo apt install nuclei")


def nuclei_submenu():
    while True:
        clear_screen()
        print(f"\n{BOLD_CYAN}Nuclei Scanner\n")
        print(f"{BOLD_GREEN}1. Basic Vulnerability Scan")
        print(f"{BOLD_GREEN}2. Moderate Scan")
        print(f"{BOLD_GREEN}3. Advanced Invasive Scan")
        print(f"{BOLD_GREEN}4. [Placeholder for future functionality]")
        print(f"{BOLD_GREEN}5. Check for Updates")
        print(f"{BOLD_RED}X. Exit Submenu")

        choice = input(f"\n{BOLD_YELLOW}Enter your choice: ").strip().lower()

        if choice == '1':
            domain = input(f"{BOLD_GREEN} Enter the domain: ").strip().lower()
            print(f"{BOLD_BLUE}Running Basic Vulnerability Scan...")
            command = f"nuclei -u https://{domain} -t {ssl_templates_path} -t {fuzzing_templates_path} -severity low,medium -o basic_scan_results.txt"
            os.system(f"qterminal -e bash -c '{command}; echo \"Press enter to close...\"; read'")
        elif choice == '2':
            domain = input(f"{BOLD_GREEN} Enter the domain: ").strip().lower()
            print(f"{BOLD_BLUE}Running Moderate Scan...")
            command = f"nuclei -u https://{domain} -t {nuclei_templates_path} -severity low,medium -o moderate_scan_results.txt"
            os.system(f"qterminal -e bash -c '{command}; echo \"Press enter to close...\"; read'")
        elif choice == '3':
            domain = input(f"{BOLD_GREEN} Enter the domain: ").strip().lower()
            print(f"{BOLD_BLUE}Running Advanced Invasive Scan...")
            command = f"nuclei -u https://{domain} -t {nuclei_templates_path} -severity high,critical -o advanced_scan_results.txt"
            os.system(f"qterminal -e bash -c '{command}; echo \"Press enter to close...\"; read'")
        elif choice == '5':
            subprocess.run(["sudo apt update && sudo apt upgrade -y"], check=True)
        elif choice == 'x':
            print(f"{BOLD_RED}Exiting Nuclei Submenu...")
            break
        else:
            print(f"{BOLD_RED}Invalid choice, please try again.")

def nuclei_main():
    clear_screen()
    if not is_go_installed():
        print(f"{BOLD_RED}Go is not installed. Installing Go...")
        install_go()
    else:
        print(f"{BOLD_GREEN}Go is already installed.")

    setup_go_environment()

    clear_screen()
    install_nuclei()
    clear_screen()
    nuclei_submenu()

if __name__ == "__main__":
    nuclei_main()



from setuptools import setup, find_packages

setup(
    name='infiltra',
    version='2.9',
    packages=find_packages(),
    install_requires=[
        'colorama',
        'pydig',
        'ascii_magic'
    ],
    entry_points={
        'console_scripts': [
            'infiltra=infiltra.ept:main',
        ],
    },
    # Include additional files into the package
    include_package_data=True,
    package_data={
        'infiltra': [
            'aort/utils/*.json',
            'version.txt',
            '*.json',
            '*.sh',
            '*.yaml',
            '*.png',
            'venv/*',
            'eyewitness/*',
            'bbot/*',
            'aort/*',
            'nuclei-templates/**/*',
        ],
    },
    # Metadata
    author='@jivy26',
    author_email='jivy26@gmail.com',
    description='CLI Based to that Automates Various Pentest Tools',
    license='MIT',
    keywords='infiltra penetration cybersecurity scanning',
)


import subprocess
import sys
import os
import random
import re

def check_install_dependencies():
    # Check for httprobe
    result = subprocess.run(['which', 'httprobe'], capture_output=True, text=True)
    if result.returncode != 0:
        print("Error: 'httprobe' is not installed. Please install it before running this script.")
        sys.exit(1)

    # Check for pup
    result = subprocess.run(['which', 'pup'], capture_output=True, text=True)
    if result.returncode != 0:
        print("Error: 'pup' is not installed. Please install it before running this script.")
        sys.exit(1)

def is_valid_domain(domain):
    # Regex for validating a domain (simple version, consider using a more complex regex for production use)
    domain_regex = re.compile(
        r'^(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,}$'
    )
    return re.match(domain_regex, domain) is not None

def run_eyewitness(input_file):
    script_directory = os.path.dirname(os.path.realpath(__file__))
    eyewitness_dir = os.path.join(script_directory, 'eyewitness')
    eyewitness_path = os.path.join(eyewitness_dir, 'Python/EyeWitness.py')

    # Check if EyeWitness.py is present
    if not os.path.exists(eyewitness_path):
        print(f"Error: EyeWitness.py not found in {eyewitness_dir}")
        sys.exit(1)

    # Determine if input is a domain, single IP, or file
    if is_valid_domain(input_file):
        # Handle domain input
        temp_output_file = f"/tmp/enum_tmp_{str(random.randint(10000, 99999))}.txt"
        command = f'curl -fsSL "https://crt.sh/?q={input_file}" | pup "td text{{}}" | grep "{input_file}" | sort -n | uniq | httprobe > {temp_output_file}'
        subprocess.run(command, shell=True, check=True)
    elif os.path.isfile(input_file):
        # Handle file input
        temp_output_file = input_file
    else:
        # Handle single IP input
        temp_output_file = f"/tmp/enum_tmp_{str(random.randint(10000, 99999))}.txt"
        with open(temp_output_file, 'w') as file:
            file.write(input_file + '\n')

    # Run EyeWitness
    print(f"Running EyeWitness on {input_file}")
    subprocess.run(["python3", eyewitness_path, "-f", temp_output_file, "--web"], check=True)

    # Cleanup the temporary file if it was created for a domain or single IP
    if temp_output_file != input_file:
        os.remove(temp_output_file)

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python3 eyewitness.py <domain or file>")
        sys.exit(1)

    input_file = sys.argv[1]
    check_install_dependencies()
    run_eyewitness(input_file)



# check_bbot.py
import subprocess

def is_bbot_installed():
    try:
        subprocess.run(["pipx", "list"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        result = subprocess.run(["pipx", "list"], capture_output=True, text=True)
        return 'bbot' in result.stdout
    except subprocess.CalledProcessError:
        # pipx is not installed or not working correctly
        return False

def install_bbot():
    try:
        subprocess.run(["pipx", "install", "bbot"], check=True)
        print("bbot has been installed successfully.")
    except subprocess.CalledProcessError as e:
        print("An error occurred while installing bbot:", e)

if __name__ == "__main__":
    if not is_bbot_installed():
        print("bbot is not installed. Installing now...")
        install_bbot()
    else:
        print("bbot is already installed.")